/*
Copyright 2007, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.

- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2007
*/

#include <ScatterWeb.System.h>
#include <ScatterWeb.CC1020.h>
#include <string.h>
#include <ScatterWeb.Time.h>

///////////////////////////////////////////////////////////
//                     variables                         //
///////////////////////////////////////////////////////////

uint16_t node;
uint16_t packetType;
uint16_t count;
bool isActive;
bool sniffer;
netpacket_send_args_t npsargs;

/**
*	Modification of Process_radioHandler.
*	Every message, that is not adressed to the attacker node and whichs
*	packet type is not filterd via the command 'packetType' will be shown
*	in a terminal.
*	Data is additionally available in its hexadecimal display.
*	
*	Used in sniffing data in the network.
*/

bool Process_radioHandler(struct netpacket_handler_args* args)
{
	/*
		Notes: * The received packet data becomes invalid when this handler is completed.
	*/
	
	// Preparations to display the data in hexadecimal also
	const uint16_t payload_size = args->payload_size;
	uint8_t* payload = (uint8_t*) args->payload;
	
	char p[payload_size * 2];
	char hexval[16] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'};	 

	// Pass packet through command interface
	if ( Cmd_radioHandler( args ) ) {
		// packet handled by messaging module
		return true;
	}

	// the receiver is not the attacker node
	// message is an unicast message
	// packet type is not filtered
	if ( args->cpaResult == ADRCOMP_OTHER && args->netheader->type != packetType) {

		// sniffer is on
		if (sniffer) {
			printf("\r\nUnicast,  \r\n");
			printf("Unicast, Einkommendes Paket von: %u \r\n", args->netheader->from);
			printf("Unicast, Einkommendes Paket an: %u \r\n", args->netheader->to);
			printf("Unicast, Pakettyp: %u \r\n", args->netheader->type);
			printf("Unicast, Payloadsize: %u \r\n", payload_size);
			printf("*** Datenbeginn *** \r\n"); 
			

			uint16_t i;
			uint16_t j;

			for (i = 0; i < payload_size; i++) {
				p[i * 2] = hexval[((*payload >> 4) & 0xF)];
				p[(i * 2) + 1] = hexval[(*payload) & 0x0F];

				printf("  %c", *payload);
				payload++;
			}
			
			printf("\r\n");

			for (j = 0; j < payload_size * 2; j++) {
				if ( j % 2 != 0) {
					printf("  ");
				}
				printf("%c", p[j]);
			}
			
			printf("\r\n*** Datenende *** \r\n");
			printf("\r\n");
		}
	
	// the receiver is not the attacker node
	// message is a broadcast message
	// packet type is not filtered
	} else if ( args->cpaResult == ADRCOMP_BROADCAST && args->netheader->type != packetType) {
		
		if (sniffer) {
			printf("\r\nBroadcast, \r\n");
			printf("Broadcast, Einkommendes Paket von: %u \r\n", args->netheader->from);
			printf("Broadcast, Einkommendes Paket an: %u \r\n", args->netheader->to);
			printf("Broadcast, Pakettyp: %u \r\n", args->netheader->type);
			printf("Broadcast, Payloadsize: %u \r\n", payload_size);
			printf("*** Datenbeginn *** \r\n"); 
			
			uint16_t i;
			uint16_t j;
			for (i = 0; i < payload_size; i++) {
				p[i * 2] = hexval[((*payload >> 4) & 0xF)];
				p[(i * 2) + 1] = hexval[(*payload) & 0x0F];

				printf("  %c", *payload);
				payload++;
			}
			
			printf("\r\n");

			for (j = 0; j < payload_size * 2; j++) {
				if ( j % 2 != 0) {
					printf("  ");
				}
				printf("%c", p[j]);
			}

			printf("\r\n*** Datenende *** \r\n");
			printf(" \r\n");
		}

	} else {
				
		// process packets for this node here
		switch ( args->netheader->type ) {
			case PING_REQUEST_PACKET:
				// send back to receiver's address
				Net_sendPong(args->netheader->from);
				break;

			case PING_REPLY_PACKET:
				LOG_LOW( "PONG from %u \r\n", args->netheader->from );
				node = args->netheader->from;
				break;
			
			case USERDEFINED_PACKET:
				
				node = args->netheader->from;
				
				LOG_LOW( "USERDEFINED_PACKET from %u \r\n", node);
				
				Net_sendArgsInit(&npsargs);
				npsargs.netheader.to = (netaddr_t) node;
				npsargs.netheader.type = USERDEFINED_PACKET;
				npsargs.netheader.flags = 0;
				Net_send(&npsargs);
				break;

			default:

				return false;
		}
		return true;				// packet handled
	}
	return false;					// packet not handled (other destination)
}

void Process_sensorHandler(void* sdata)
{
}


/**
*	Initializazion of variables.
*/
void Process_init()
{
	System_registerCallback( C_RADIO, ( fp_vp )Process_radioHandler );
	count = 1;
	isActive = false;
	packetType = -1;
	sniffer = true;
	System_registerCallback( C_SENSOR, ( fp_vp )Process_sensorHandler );
}

/**
 * @brief	Ping request
 *
 * Send a ping request to another node
 * Example: ping 24
 */
COMMAND( ping, 0, cmdargs )
{

	node = String_parseUint16( cmdargs->args, NULL );
	if ( node > 0 ) {
		Net_sendPing((netaddr_t)node);
	}
}

/**
 * @brief Echo Arguments
 *
 * Testcase 1: long command name
 * Testcase 2: argument splitting
 */
COMMAND( echoargs, CMDFLAG_SERIAL, cmdargs )
{
	uint16_t i;
	uint16_t count = Cmd_SplitArgs( cmdargs->args );
	char* arg = (char*) cmdargs->args;

	for ( i = 1; i <= count; i++ ) {
		printf("%u: %*s\n", i, strlen(arg), arg);
		arg = ( char* )Cmd_GetNextArg( arg );
	}
	snprintf(cmdargs->response_args, cmdargs->response_bufsize, "%u args found", count );
}

// Software header.
APP(
    APPH_STATIC_APP,	// Application Type
    __APP_NAME,			// Application Name (8 chars)
    1, 0				// Aplication Version (2 bytes)
);

/**
*	This void sets up a timer and sends data packets as long as 
*	the command 'startSend' was executed.
*	It does not send data, when the command 'stopSend' is executed'.
*	Used in occupying the medium.
*/
void callMe()
{
	if (isActive) {
		//printf("+++ Der boese Junge sendet wieder +++ %u \r\n", count);
		
		//printf(message);
		
		netpacket_send_args_t npsargs;
		Net_sendArgsInit(&npsargs);
		npsargs.netheader.to = (netaddr_t) 254;
		npsargs.netheader.type = USERDEFINED_PACKET;
		npsargs.netheader.flags = 0;
		
		npsargs.payload[0].buffer = "+++ Der boese Junge sendet wieder +++";
		npsargs.payload[0].size = 37;
		
		Net_send(&npsargs);
		Timers_add(5, callMe, 0xFFFF);
	}
}

/**
*	Simple command to start sending data.
*	Used to occupy the medium.
*/
COMMAND( startSend, CMDFLAG_SERIAL, cmdargs )
{
	isActive = true;
	printf("*************** \r\n");
	printf("Sending enabled \r\n", count);
	printf("*************** \r\n");
	callMe();
}

/**
*	Simple command to stop sending data.
*	Used in occupying the medium.
*/
COMMAND( stopSend, CMDFLAG_SERIAL, cmdargs )
{
	isActive = false;
	count = 1;
	printf("*************** \r\n");
	printf("Sending stopped \r\n", count);
	printf("*************** \r\n");
}

/**
*	Command to occupy the medium. View report for more information.
*/
//COMMAND( BLG_M, CMDFLAG_SERIAL, cmdargs )
//{
//}

/**
*	Simple command to disable the watch dog.
*	Used in occupying the medium.
*/
COMMAND( WDStop, CMDFLAG_SERIAL, cmdargs )
{
	System_stopWatchdog();
}

/**
*	Simple command to enable the watch dog.
*	Used in occupying the medium.
*/
COMMAND( WDStart, CMDFLAG_SERIAL, cmdargs )
{
	System_startWatchdog();
}

/**
*	Simple command to filter the given packt type in order not to log the 
*	mass of 'Hello'-messages.
*	@param      packetType  Packet Type, that has to be filtered.
*/
COMMAND( packetType, CMDFLAG_SERIAL, cmdargs )
{
	packetType = String_parseUint16( cmdargs->args, NULL );
}

/**
*	Simple command to start the sniffer.
*/
COMMAND( startSniffer, CMDFLAG_SERIAL, cmdargs )
{
	sniffer = true;
}

/**
*	Simple command to stop the sniffer.
*/
COMMAND( stopSniffer, CMDFLAG_SERIAL, cmdargs )
{
	sniffer = false;
}