/*
Copyright 2006, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2006
*/

#include "ScatterWeb.Logger.h"

// logFirstAddress
#define logFirstAddress   (0x1000)

// logLastAddress
#define logLastAddress    (0xFFFF)

// logPackageSizeOf
#define logPackageSizeOf  (0x000C)

//  epromWritePointer
UINT16 eepromWritePointer; 

//  eepromWritePointer
UINT16 eepromReadPointer;

//  eepromRAWFlag
UINT8 eepromRAWFlag;

//  requestSensorMask
UINT8 requestSensorMask;

//  requestCommandType
UINT8 requestCommandType;

//  period for rld 02/04
btime_t btime;      
time_t time_from;
time_t time_to;

//  Global variabel for rld
UINT16 sendDataCount;
UINT16 sendDataRP;
UINT16 sendDataWP;
UINT8 sendDataRAWF;

//  data packages
char temp_data[12];
sdata_t* sdata_ptr;

/**
*  (data Logger)
*  data_t  - packet data
*  id      - packet number
*/
typedef struct {
  sdata_t data;
  UINT16 id;
} LogDataPacketStruct;

void _Logger_sendData(UINT8 succsess);

// funktion for sending requested data package with Net_send.
//
//  build logDataPacket (lDP) that shall be send with Net_send. Was sending
//  unsuccsessfull, then the function sendData will be called with the parameter
//  false, otherwise true.
void _Logger_sendLogPacket(char* data, int id) {
  System_startWatchdog();
  sdata_t* send_sdata_t;
  LogDataPacketStruct LogDataPacket;
  // build data packet
  send_sdata_t=(sdata_t*) data;
  LogDataPacket.id = id;
  LogDataPacket.data.id = send_sdata_t->id;
  LogDataPacket.data.sensor = send_sdata_t->sensor;
  LogDataPacket.data.timeStamp = send_sdata_t->timeStamp;
  LogDataPacket.data.value = send_sdata_t->value;
  // build send packet
  packet_t p;
  p.to = BROADCAST;						//set the destination address to broadcast
  p.type = LOGGER_PACKET;				//set the packet type to test packet
  p.header = 0;							//dont use an optional second header
  p.header_length = 0;					//dont use an optional second header
  p.data = (UINT8*) &LogDataPacket;		//pass the beginning of the paylod
  p.data_length = sizeof(LogDataPacket);//give the length of the payload  
  // send packet
  Net_send(&p, _Logger_sendData);
}

// funktion for searching requested data packages.
//
//  The sendData compares each valid loged data package. Therefore exists the global
//	variables sendDataRP (eWP), sendDataWP (eWP), sendDataRAWF (eRAWF), time_from,
//  time_to, requestCommandType (rCT), and requestSensorMask. Those secpify the intervall
//  (eRP,eWP,ERAWF), the period (time_from, time_to), and sensortypes of requested
//  data. Those variables had to be set before calling sendData.
//
//  The procedure goes linear through the commited data intervall and search for an
//  data, that match the commited sensortype. In case of rCT equal to "05" additional
//  compares if the timestamp of data match in the commited period.
//  
//  If there is an requested data package, sendData calls sendLogPacket. As long as
//  data was send unsuccsesfull sendData calls sendLogPacket with same package. otherwise
//  sendData search for the next requestest package.
void _Logger_sendData(UINT8 succsess) {
  char temp_data[12];
  sdata_t* sdata_ptr;
  // last package was succsessfull send, increasing count/eRP
  if (succsess) {
    sendDataCount++;
    sendDataRP+=logPackageSizeOf;
      // handle ring buffer
    if (sendDataRP>=logLastAddress-logPackageSizeOf) sendDataRP=logFirstAddress;
    if (sendDataRP==sendDataWP) sendDataRAWF=0;
  }
  // only sensortype have to match with request
  if (requestCommandType==04) {
    // linear search
    while((sendDataRP<sendDataWP)||sendDataRAWF) {
      IO_read(sendDataRP,(char*) &temp_data, 12);
	  // request package found
      if (((UINT8)temp_data[2]&requestSensorMask)) {
        _Logger_sendLogPacket((char*)temp_data, sendDataCount);
        break;
      }
	  // continue, because last package didnt match request
      else {
        sendDataRP+=logPackageSizeOf;
          // handle ring buffer
        if (sendDataRP>=logLastAddress-logPackageSizeOf) sendDataRP=logFirstAddress;
        if (sendDataRP==sendDataWP) sendDataRAWF=0;
      }
    }
  // sensortype and period have to match with request
  } else if (requestCommandType==05) {
     // linear search
    while((sendDataRP<sendDataWP)||sendDataRAWF) {
      IO_read(sendDataRP,(char*) &temp_data, 12);
      if (((UINT8)temp_data[2]&requestSensorMask)) {
        // sensortype match
        sdata_ptr=(sdata_t*)temp_data;
        if (((time_from.secs)<=(sdata_ptr->timeStamp).secs) && ((time_to.secs)>=(sdata_ptr->timeStamp).secs)) {
          // period match, send package
          _Logger_sendLogPacket((char*)temp_data, sendDataCount);
          break;
        }
		else {
          // continue, because last package didnt match request
          sendDataRP+=logPackageSizeOf;
          // handle ring buffer
          if (sendDataRP>=logLastAddress-logPackageSizeOf) sendDataRP=logFirstAddress;
          if (sendDataRP==sendDataWP) sendDataRAWF=0;
        }
      }
    }       
  }
}

//  dld <br> Delete all log data (eepromReadPointer=eepromWritePointer).
COMMAND(dld, 0){
  eepromReadPointer=eepromWritePointer;
  eepromRAWFlag=0;
}

// rld <br> Main funtion to readouts the log data.
// Parameter:
//
//  rld ii [jj] [YY-MM-TT HH:MM:SS YY-MM-TT HH:MM:SS]
//  
//    ii : specify the type of command
//  
//     00 : Readout the count of all logged data.
//     01 : Readout the count of data, with a specify sensor mask.
//     02 : Readout the count of data, with a specify sensor mask and over a specific period.
//     03 : Readout all data from the eeprom to the serial output.
//     04 : Readout data with a specific sensor mask with Net_Send.
//     05 : Readout data with specific sensor mask and period with Net_Send.
//  
//    jj : specific sensor mask. necessary for rld 01/02/03/04/05.
//  
//    YY-MM-TT HH:MM:SS YY-MM-TT HH:MM:SS : necessary for rld 02/04 (FROM TO).

COMMAND(rld, 0){
  // reqads period (global)
  Time_readTime(&str[11], &btime);
  Time_convertTime(&btime, &time_from);
  Time_readTime(&str[28], &btime);
  Time_convertTime(&btime, &time_to);
  //  create tmp. pointer for eeprom (eRP eRAWF)
  UINT16 tmp_rp=eepromReadPointer;
  UINT8 tmp_rf=eepromRAWFlag; 
  //  counter for data package
  UINT16 count;
  //  reads ii (global)
  requestCommandType = String_fromHexToByte( &str[4]);
  requestSensorMask = String_fromHexToByte( &str[7]);
  //  switch for specified command (ii)
  switch (requestCommandType){
    case 00:
      if (!eepromRAWFlag) count=(eepromWritePointer-eepromReadPointer)/12;
      else count=((logLastAddress-eepromReadPointer)+(eepromWritePointer-logFirstAddress))/logPackageSizeOf;
      String_write(term_reply, "[rld] 00 - %i",count);      
      break;
    case 01:
      System_stopWatchdog();
      count=0;
	  // linear search for requestet data packages
      while ((tmp_rp<eepromWritePointer)||tmp_rf) {
        IO_read(tmp_rp,(char*) &temp_data, 12);
        tmp_rp+=logPackageSizeOf;
        if (tmp_rp>=logLastAddress-logPackageSizeOf) tmp_rp=logFirstAddress;
        if (tmp_rp==eepromWritePointer) tmp_rf=0;
		// compares the sensortype with jj
        if (((UINT8)temp_data[2]&requestSensorMask)) count++;
      }  
      System_startWatchdog();
      String_write(term_reply, "[rld] 01 - %i" , count);
      break;
    case 02:
      System_stopWatchdog();      
      count=0;
	  // linear search for requestet data packages
      while ((tmp_rp<eepromWritePointer)||tmp_rf) {
          IO_read(tmp_rp,(char*) &temp_data, 12);
          tmp_rp+=logPackageSizeOf;
          if (tmp_rp>=logLastAddress-logPackageSizeOf) tmp_rp=logFirstAddress;
          if (tmp_rp==eepromWritePointer) tmp_rf=0;
		  // compares the sensortype with jj
          if (((UINT8)temp_data[2]&requestSensorMask)) {
		    // compares the data timestamp with given period 
            sdata_ptr=(sdata_t*)temp_data;
            if ((time_from.secs)<=(sdata_ptr->timeStamp).secs) 
              if ((time_to.secs)>=(sdata_ptr->timeStamp).secs) 
                count++;
          }
      }
      System_startWatchdog();
      String_write(term_reply, "[rld] 02 - %i",count);
      break;
    case 03:
      count=1;
      System_stopWatchdog();
      while ((tmp_rp<eepromWritePointer)||tmp_rf) {
        IO_read(tmp_rp,(char*) &temp_data, 12);
        tmp_rp+=logPackageSizeOf;
        if (tmp_rp>=logLastAddress-logPackageSizeOf) tmp_rp=logFirstAddress;
        if (tmp_rp==eepromWritePointer) tmp_rf=0;
        Comm_print("LOG: %i: %e\r\n", count, (sdata_t*)temp_data);
        count++;
      }
      System_startWatchdog();
	  String_write(term_reply, "[rld] 03");
      break;      
    case 04:
	  // sets global variable for request
      sendDataCount=1;
      sendDataRP=eepromReadPointer;
      sendDataWP=eepromWritePointer;
      sendDataRAWF=eepromRAWFlag;
      // starts linear go through (with null because no data send until now. last send was unsuccesfull).
      _Logger_sendData(0);
      String_write(term_reply, "[rld] 04");
      break;
    case 05:
      // sets global variable for request
      sendDataCount=1;
      sendDataRP=eepromReadPointer;
      sendDataWP=eepromWritePointer;
      sendDataRAWF=eepromRAWFlag;
      // starts linear go through (with null because no data send until now. last send was unsuccesfull).
      _Logger_sendData(0); 
      String_write(term_reply, "[rld] 05");
      break;
  }
}

bool Logger_init() {
  eepromWritePointer=logFirstAddress;
  eepromReadPointer=logFirstAddress;
  eepromRAWFlag=0;
  return True;
}

bool Logger_write(sdata_t* data) {
  // overwrite the oldest data on eeprom if eeprom full
  if (eepromRAWFlag&&(eepromWritePointer==eepromReadPointer)) eepromReadPointer+=logPackageSizeOf;
  //writes data to actual eWP and sets eWP to next data position
  IO_write(eepromWritePointer, (UINT8*) data, sizeof(*data));
  eepromWritePointer+=logPackageSizeOf;
  //if the eWP leaves the valid log interval of the eeprom, sets back to the first address
  if (eepromWritePointer>=(logLastAddress-logPackageSizeOf)) {
    //if the eeprom is full, it is necessary to set the eRP to the lFA, too
    if (eepromRAWFlag&&(eepromWritePointer==eepromReadPointer)) eepromReadPointer=logFirstAddress;
    eepromWritePointer=logFirstAddress;
    eepromRAWFlag=1;
  }
  return True;
}
