/*
Copyright 2007, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2007
*/

/**
 * @file			ScatterWeb.ADDA.internals.h
 * @ingroup	libADDA
 * @brief			Access functions for the AD/DA converter
 * 
 * @author		Bastian Blywis
 * @date			Mi Okt 31 11:59:35 2007
 * @since		1.2
 * @version	$Revision$
 * 
 * Header file containing private declarations used by the ADDA library.
 */

#ifndef ADDA_INT_H_
#define ADDA_INT_H_

#include <ScatterWeb.System.h>

/**
 * @brief Converts bits to voltage.
 *
 * The given value has to be converted with VeRef = +Vcc as reference.
 * Conversion uses always the current value of VeRef.
 * @param bits number to convert to U
 * @return U/[V]
 */
float ADDA_bits2Volt(uint16_t bits);

/**
 * @brief Print a floating point value
 *
 * MSPGCC lacks float in- and output support. This functions
 * lets you at least print a float to the stdout.<br>
 * Conversion to ASCII is done by casting f to uint16_t to get
 * the pre-decimal positions. The post-decimal positions are obtained
 * by a multiplication by 1000.
 * @param f float to be printed
 */
void ADDA_printFloat(float f);

float ADDA_parseFloat(char* a, uint8_t* b);

#endif /*ADDA_INT_H_*/
