/*
Copyright 2007, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2007
*/

/**
 * @file			ScatterWeb.MD2.h
 * @ingroup	libMD2
 * @brief			MD2 library, Public interface
 *
 * @author		Harald Sauff
 * @date			Aug 2007
 * @since 		1.2
 * @version	$Revision$
 *
 * <p>
 * Copyright (C) 1990-2, RSA Data Security, Inc. Created 1990. All
 * rights reserved.
 * <p>
 * License to copy and use this software is granted for
 * non-commercial Internet Privacy-Enhanced Mail provided that it is
 * identified as the "RSA Data Security, Inc. MD2 Message Digest
 * Algorithm" in all material mentioning or referencing this software
 * or this function.
 * <p>
 * RSA Data Security, Inc. makes no representations concerning either
 * the merchantability of this software or the suitability of this
 * software for any particular purpose. It is provided "as is"
 * without express or implied warranty of any kind.
 * <p>
 * These notices must be retained in any copies of any part of this
 * documentation and/or software.
 * <p>
 * - source code taken from reference implementation in RFC 1319
 * - adapted by Harald Sauff,
 *   Technical University of Hamburg-Harburg (TUHH)
 *
 */

/**
 * @ingroup	libMD2
 * @{
 */

#ifndef __MD2_H__
#define __MD2_H__

#include <ScatterWeb.System.h>

/******************************************************************************
 * @name	 initialisation and usage
 * @{
 */

/**
 * @brief MD2 checksum calculation
 *
 * Generates a MD2 checksum
 *
 * @param input		pointer to input to generate a MD2 checksum of
 * @param length	length of input
 * @param output	output of the MD2 checksum
 * @return					Pointer of output.
 * @since                 	1.2
 * @version 				$Revision$
 */
uint8_t* MD2_get(uint8_t *input, uint16_t length, uint8_t *output);

/**
 * @brief Required dummy function
 */
void libMD2_Init() {}

#endif /*__MD2_H__*/
/** @} */
