/**
 * @file	AppConf.h
 * @brief	Application Configuration
 * 
 * Put all defines that configure application and libraries into this file.
 * 
 * This configuration is automatically injected into the application and all
 * included libraries during the build process.
 */

#ifndef APPCONF_H_
#define APPCONF_H_

#define __SCATTERWEB__

#define SCATTERWEB_BLD			1
#define LOADER_WITH_PRINTF		1

#define SD_LED_READ_ON		P5OUT &=~0x80
#define SD_LED_READ_OFF		P5OUT |= 0x80
#define SD_LED_WRITE_ON		SD_LED_READ_ON
#define SD_LED_WRITE_OFF	SD_LED_READ_OFF

#define FAT12_SUPPORT			0						///< Support for 6-Bit FAT12 entries
#define	FAT_STRICT_TYPE_CHECK	0
#define FAT_WRITE				1
#define FAT_FILE_HELPER			FAT_WRITE				///< Easy file handle manipulation
#define FAT_FIND_SIMPLE			1						///< File find function

#define SD_READ_BYTE			0						///< Read-function optimized for single bytes
#define	SD_READ_ANY				1						///< Read-function for any size
#define SD_INFO					0						///< Functions for reading and interpreting card registers
#define SD_WRITE				FAT_WRITE
#define SD_CACHE				1
#define SD_FIND_FILE			0
#define SD_ERASE				0

#define SPI_WRITE				SD_WRITE
#define SPI_DMA_READ			0						///< Speed up read for ~50 bytes code size increase
#define SPI_DMA_WRITE			0						///< Speed up write for ~50 bytes code size increase

#endif /*APPCONF_H_*/
